@@ -9,18 +9,18 @@ Provides backends that deal with local filesystem access.
 from __future__ import with_statement
 
 from contextlib import contextmanager
+import dbm
 import os
-import pickle
 import threading
 
-from ..api import CacheBackend
+from ..api import BytesBackend
 from ..api import NO_VALUE
 from ... import util
 
 __all__ = ["DBMBackend", "FileLock", "AbstractFileLock"]
 
 
-class DBMBackend(CacheBackend):
+class DBMBackend(BytesBackend):
     """A file-backend using a dbm file to store keys.
 
     Basic usage::
@@ -157,9 +157,6 @@ class DBMBackend(CacheBackend):
             util.KeyReentrantMutex.factory,
         )
 
-        import dbm
-
-        self.dbmmodule = dbm
         self._init_dbm_file()
 
     def _init_lock(self, argument, suffix, basedir, basefile, wrapper=None):
@@ -183,7 +180,7 @@ class DBMBackend(CacheBackend):
                     exists = True
                     break
         if not exists:
-            fh = self.dbmmodule.open(self.filename, "c")
+            fh = dbm.open(self.filename, "c")
             fh.close()
 
     def get_mutex(self, key):
@@ -213,53 +210,50 @@ class DBMBackend(CacheBackend):
     @contextmanager
     def _dbm_file(self, write):
         with self._use_rw_lock(write):
-            dbm = self.dbmmodule.open(self.filename, "w" if write else "r")
-            yield dbm
-            dbm.close()
+            with dbm.open(self.filename, "w" if write else "r") as dbm_obj:
+                yield dbm_obj
 
-    def get(self, key):
-        with self._dbm_file(False) as dbm:
-            if hasattr(dbm, "get"):
-                value = dbm.get(key, NO_VALUE)
+    def get_serialized(self, key):
+        with self._dbm_file(False) as dbm_obj:
+            if hasattr(dbm_obj, "get"):
+                value = dbm_obj.get(key, NO_VALUE)
             else:
                 # gdbm objects lack a .get method
                 try:
-                    value = dbm[key]
+                    value = dbm_obj[key]
                 except KeyError:
                     value = NO_VALUE
-            if value is not NO_VALUE:
-                value = pickle.loads(value)
             return value
 
-    def get_multi(self, keys):
-        return [self.get(key) for key in keys]
+    def get_serialized_multi(self, keys):
+        return [self.get_serialized(key) for key in keys]
 
-    def set(self, key, value):
-        with self._dbm_file(True) as dbm:
-            dbm[key] = pickle.dumps(value, pickle.HIGHEST_PROTOCOL)
+    def set_serialized(self, key, value):
+        with self._dbm_file(True) as dbm_obj:
+            dbm_obj[key] = value
 
-    def set_multi(self, mapping):
-        with self._dbm_file(True) as dbm:
+    def set_serialized_multi(self, mapping):
+        with self._dbm_file(True) as dbm_obj:
             for key, value in mapping.items():
-                dbm[key] = pickle.dumps(value, pickle.HIGHEST_PROTOCOL)
+                dbm_obj[key] = value
 
     def delete(self, key):
-        with self._dbm_file(True) as dbm:
+        with self._dbm_file(True) as dbm_obj:
             try:
-                del dbm[key]
+                del dbm_obj[key]
             except KeyError:
                 pass
 
     def delete_multi(self, keys):
-        with self._dbm_file(True) as dbm:
+        with self._dbm_file(True) as dbm_obj:
             for key in keys:
                 try:
-                    del dbm[key]
+                    del dbm_obj[key]
                 except KeyError:
                     pass
 
 
-class AbstractFileLock(object):
+class AbstractFileLock:
     """Coordinate read/write access to a file.
 
     typically is a file-based lock but doesn't necessarily have to be.
